#include "editor.h"
#include "bridge.h"
#include "../common/sources/system/cgraphics2d.h"
#include "../common/sources/system/filecache.h"
#include "tools.h"
#include "rectangles.h"
#include <QDebug>
#include <QtXml/qdom.h>
#include <QFile>
#include <QQmlApplicationEngine>

CEditor  editor;
extern QQmlApplicationEngine*  myQmlEngine;
extern QQuickItem*  tileSelGrid;
extern QQuickItem*  wallSelGrid;
extern QQuickItem*  objSelGrid;

//---------------------------------------------------------------------------------------------
CEditor::CEditor()
{
    mUndoStack = new QUndoStack();
    mSelectStart.x = -1;
}

//---------------------------------------------------------------------------------------------
// get the wall side for a position relative to a tile
EWallSide   CEditor::getWallSide(CVec2 localPos)
{
    bool side1 = (localPos.y < localPos.x);
    bool side2 = (localPos.y < TILE_SIZE - 1 - localPos.x);

    if (side1 == true && side2 == true)
        return eWallSideUp;

    if (side1 == true && side2 == false)
        return eWallSideRight;

    if (side1 == false && side2 == true)
        return eWallSideLeft;

    return eWallSideDown;
}

//---------------------------------------------------------------------------------------------
// get the wall side for the mouse position in the map
EWallSide   CEditor::getWallSideAbs(CVec2 pos)
{
    return getWallSide(pos % TILE_SIZE);
}

//---------------------------------------------------------------------------------------------
CRect CEditor::getSelectionRect()
{
    CRect   rect = getSelectionRectPixel();

    rect.tl /= TILE_SIZE;
    rect.br /= TILE_SIZE;
    return rect;
}

//---------------------------------------------------------------------------------------------
CRect CEditor::getSelectionRectPixel()
{
    CRect   rect;

    // handle the cases when we do not draw the rectangle from top/left to bottom/right
    rect.tl.x = MIN(mSelectStart.x, mSelectEnd.x);
    rect.tl.y = MIN(mSelectStart.y, mSelectEnd.y);
    rect.br.x = MAX(mSelectStart.x, mSelectEnd.x);
    rect.br.y = MAX(mSelectStart.y, mSelectEnd.y);
    return rect;
}


//---------------------------------------------------------------------------------------------
// returns the string that tells the position of the cursor in the map
QString CEditor::getPositionString(CVec2 mousePos, ETabIndex tabIndex)
{
    CVec2 tilePos = mousePos / TILE_SIZE;
    static char tempString[256];
    tempString[0] = 0;

    if (tabIndex == eTabTiles)
    {
        sprintf(tempString, "(%d,%d)", tilePos.x, tilePos.y);
    }
    else if (tabIndex == eTabWalls)
    {
        EWallSide   side = getWallSideAbs(mousePos);

        switch (side)
        {
            case eWallSideUp:
                sprintf(tempString, "(%d,%d) up", tilePos.x, tilePos.y);
            break;

            case eWallSideDown:
                sprintf(tempString, "(%d,%d) down", tilePos.x, tilePos.y);
            break;

            case eWallSideLeft:
                sprintf(tempString, "(%d,%d) left", tilePos.x, tilePos.y);
            break;

            case eWallSideRight:
                sprintf(tempString, "(%d,%d) right", tilePos.x, tilePos.y);
            break;

            default:
            break;
        }
    }
    else if (tabIndex == eTabObjects)
    {
        CVec2   objPos = (mousePos % TILE_SIZE) / (TILE_SIZE / 2);
        sprintf(tempString, "(%d,%d) (%d,%d)", tilePos.x, tilePos.y, objPos.x, objPos.y);
    }

    return QString(tempString);
}

//---------------------------------------------------------------------------------------------
void   CEditor::hideSelectionInfos()
{
    bridge.set_tileSelScrollVis(false);
    bridge.set_wallSelScrollVis(false);
    bridge.set_objectSelScrollVis(false);
    bridge.set_rectSelScrollVis(false);
}

//---------------------------------------------------------------------------------------------
void CEditor::displayWall(QImage* image, CVec2 pos, EWallSide side, int angle)
{
    int type = map.getTile(pos)->mWalls[side].getType();
    QString file = wallsDatas[type].image;

    if (file.isEmpty() == false)
    {
        QImage  tileImage = fileCache.getImage(file.toLocal8Bit().constData());
        CVec2   pos2 = pos * TILE_SIZE;
        graph2D.drawImage(image, pos2, tileImage, angle);
    }
}

//---------------------------------------------------------------------------------------------
QImage CEditor::displayMap(CMap& map, CVec2 mousePos, ETabIndex tabIndex)
{
    QImage image(map.mSize.x * TILE_SIZE,
                 map.mSize.y * TILE_SIZE,
                 QImage::Format_RGBA8888);

    image.fill(QColor(0, 0, 0));

    if (map.isLoaded() == true)
    {
        // draw tiles
        for (int y = 0; y < map.mSize.y; ++y)
            for (int x = 0; x < map.mSize.x; ++x)
            {
                CVec2   pos(x, y);
                CVec2   pos2 = pos * TILE_SIZE;

                int type = map.getTile(pos)->getType();
                QString file = tilesDatas[type].image;

                if (file.isEmpty() == false)
                {
                    QImage  tileImage = fileCache.getImage(file.toLocal8Bit().constData());
                    graph2D.drawImage(&image, pos2, tileImage);
                }
            }

        // draw walls
        for (int y = 0; y < map.mSize.y; ++y)
            for (int x = 0; x < map.mSize.x; ++x)
            {
                CVec2   pos(x, y);

                displayWall(&image, pos, eWallSideUp, 0);
                displayWall(&image, pos, eWallSideDown, 180);
                displayWall(&image, pos, eWallSideLeft, -90);
                displayWall(&image, pos, eWallSideRight, 90);
            }

        // draw objects
        for (int y = 0; y < map.mSize.y * 2; ++y)
            for (int x = 0; x < map.mSize.x * 2; ++x)
            {
                CVec2   pos(x, y);

                if (map.findObjectsStack(pos, eWallSideMax) != NULL)
                {
                    QImage  objImage = fileCache.getImage("tiles/Object.png");
                    CVec2   pos2 = pos * (TILE_SIZE / 2);
                    graph2D.drawImage(&image, pos2, objImage);
                }
            }
    }

    rectangles.drawSelection(&image, tabIndex);
    rectangles.drawCursor(&image, mousePos, tabIndex);
    rectangles.drawPasteSelection(&image);

    return image;
}

//---------------------------------------------------------------------------------------------
void    CEditor::useTool()
{
    switch (bridge.mTool)
    {
    case eQMLToolDraw:
        toolDraw();
        break;

    case eQMLToolSelect:
        toolSelect();
        break;

    case eQMLToolPaste:
        toolPaste();
        break;
    }
}

//---------------------------------------------------------------------------------------------
void    CEditor::toolDraw()
{
    if (bridge.rightPressed == true || bridge.leftPressed == true)
    {
        if (bridge.mTabIndex == eTabTiles)
        {
            // tiles
            drawTileTool*   tool = new drawTileTool();
            int type = 0;

            if (bridge.rightPressed == true)
                type = 0;
            else if (bridge.leftPressed == true)
                type = bridge.get_tileType();

            if (tool->init(bridge.mMousePos, type) == true)
            {
                unselect();
                mUndoStack->push(tool);
            }
            else
                delete tool;
        }
        else if (bridge.mTabIndex == eTabWalls)
        {
            // walls
            drawWallTool*   tool = new drawWallTool();
            int type = 0;

            if (bridge.rightPressed == true)
                type = 0;
            else if (bridge.leftPressed == true)
                type = bridge.get_wallType();

            if (tool->init(bridge.mMousePos, type) == true)
            {
                unselect();
                mUndoStack->push(tool);
            }
            else
                delete tool;
        }
        else if (bridge.mTabIndex == eTabObjects)
        {
            // objects
            if (bridge.leftPressing == true)
            {
                addObjectTool*   tool = new addObjectTool();

                if (tool->init(bridge.mMousePos, bridge.get_objectType()) == true)
                {
                    unselect();
                    mUndoStack->push(tool);
                }
                else
                    delete tool;
            }
        }
    }
}

//---------------------------------------------------------------------------------------------
// Did we select a single wall ?
bool   CEditor::isSelectSingleWall()
{
    CVec2   startPos = mSelectStart / TILE_SIZE;
    CVec2   endPos   = mSelectEnd   / TILE_SIZE;
    EWallSide   startSide = getWallSideAbs(mSelectStart);
    EWallSide   endSide   = getWallSideAbs(mSelectEnd);

    return (startPos == endPos && startSide == endSide);
}

//---------------------------------------------------------------------------------------------
// Did we select a single object ?
bool   CEditor::isSelectSingleObject()
{
    CVec2   startPos = mSelectStart / (TILE_SIZE / 2);
    CVec2   endPos   = mSelectEnd   / (TILE_SIZE / 2);

    return (startPos == endPos);
}

//---------------------------------------------------------------------------------------------
void        CEditor::copyData(CRect rect)
{
    CVec2   size = (rect.br / TILE_SIZE) - (rect.tl / TILE_SIZE) + CVec2(1, 1);
    mClipboard.copyFromMap(rect.tl, size, bridge.mTabIndex);
}

//---------------------------------------------------------------------------------------------
void        CEditor::toolCopy()
{
    if (mSelectStart.x != -1)
    {
        CRect   rect = getSelectionRectPixel();
        copyData(rect);
        unselect();
    }
}

//---------------------------------------------------------------------------------------------
void        CEditor::toolCut()
{
    if (mSelectStart.x != -1)
    {
        cutTool*   tool = new cutTool();
        CRect   rect = getSelectionRectPixel();
        copyData(rect);

        if (tool->init(mSelectStart, bridge.mTabIndex) == true)
            mUndoStack->push(tool);
        else
            delete tool;
    }
    unselect();
}

//---------------------------------------------------------------------------------------------
void        CEditor::toolPaste()
{
    if (bridge.leftPressing == true)
    {
        unselect();
        pasteTool*   tool = new pasteTool();

        if (tool->init(bridge.mMousePos, bridge.mTabIndex) == true)
            mUndoStack->push(tool);
        else
            delete tool;
    }
}

//---------------------------------------------------------------------------------------------
void    CEditor::unselect()
{
    mSelectStart.x = -1;
    bridge.updateQMLImage();
}

//---------------------------------------------------------------------------------------------
// destroy the additionnal parameters in the selection info groupBox
void    CEditor::deleteParamItems(std::vector<QQuickItem*>* list)
{
    for (size_t i = 0; i < list->size(); ++i)
        delete (*list)[i];
    list->clear();
}

//---------------------------------------------------------------------------------------------
// creates the additionnal parameters in the selection info groupBox
void    CEditor::addTileParamsItems(uint8_t type)
{
    CVec2   pos = mSelectStart / TILE_SIZE;
    CTile*  tile = map.getTile(pos);

    std::vector<CParamType>& sourceList = CMap::mTilesParams[type];

    for (size_t i = 0; i < sourceList.size(); ++i)
    {
        CParam* param = tile->mParams[i];

        if (sourceList[i].mType == eParamOrnate)
        {
            CParamOrnate*   par = (CParamOrnate*)param;
            addLabel(tileSelGrid, sourceList[i].mName + ":", &mTileParamItems);
            addComboBox(tileSelGrid, bridge.floorOrnatesList, &mTileParamItems, i, par->mValue);
        }
        else if (sourceList[i].mType == eParamDoor)
        {
            CParamDoor*   par = (CParamDoor*)param;
            addLabel(tileSelGrid, sourceList[i].mName + ":", &mTileParamItems);
            addComboBox(tileSelGrid, bridge.doorsList, &mTileParamItems, i, par->mValue);
        }
        else if (sourceList[i].mType == eParamDoorOrnate)
        {
            CParamDoorOrnate*   par = (CParamDoorOrnate*)param;
            addLabel(tileSelGrid, sourceList[i].mName + ":", &mTileParamItems);
            addComboBox(tileSelGrid, bridge.doorOrnatesList, &mTileParamItems, i, par->mValue);
        }
        else if (sourceList[i].mType == eParamEnum)
        {
            CParamEnum*   par = (CParamEnum*)param;
            addLabel(tileSelGrid, sourceList[i].mName + ":", &mTileParamItems);
            addComboBox(tileSelGrid, sourceList[i].mValues, &mTileParamItems, i, par->mValue);
        }
        else if (sourceList[i].mType == eParamInt)
        {
            CParamInt*   par = (CParamInt*)param;
            addLabel(tileSelGrid, sourceList[i].mName + ":", &mTileParamItems);
            addTextField(tileSelGrid, QString::number(par->mValue), &mTileParamItems, i);
        }
        else if (sourceList[i].mType == eParamBool)
        {
            CParamBool*   par = (CParamBool*)param;
            addLabel(tileSelGrid, sourceList[i].mName + ":", &mTileParamItems);
            addCheckBox(tileSelGrid, par->mValue, &mTileParamItems, i);
        }
        else if (sourceList[i].mType == eParamScript)
        {
            CParamScript*   par = (CParamScript*)param;
            addLabel(tileSelGrid, sourceList[i].mName + ":", &mTileParamItems);
            addScriptField(tileSelGrid, QString(par->mValue.c_str()), &mTileParamItems, i);
        }
    }
}

//---------------------------------------------------------------------------------------------
// creates the additionnal parameters in the selection info groupBox
void    CEditor::addWallParamsItems(uint8_t type)
{
    CVec2       pos = mSelectStart / TILE_SIZE;
    EWallSide   side = getWallSideAbs(mSelectStart);
    CWall*      wall = &map.getTile(pos)->mWalls[side];

    std::vector<CParamType>& sourceList = CMap::mWallsParams[type];

    for (size_t i = 0; i < sourceList.size(); ++i)
    {
        CParam* param = wall->mParams[i];

        if (sourceList[i].mType == eParamOrnate)
        {
            CParamOrnate*   par = (CParamOrnate*)param;
            addLabel(wallSelGrid, sourceList[i].mName + ":", &mWallParamItems);
            addComboBox(wallSelGrid, bridge.ornatesList, &mWallParamItems, i, par->mValue);
        }
        else if (sourceList[i].mType == eParamInt)
        {
            CParamInt*   par = (CParamInt*)param;
            addLabel(wallSelGrid, sourceList[i].mName + ":", &mWallParamItems);
            addTextField(wallSelGrid, QString::number(par->mValue), &mWallParamItems, i);
        }
        else if (sourceList[i].mType == eParamBool)
        {
            CParamBool*   par = (CParamBool*)param;
            addLabel(wallSelGrid, sourceList[i].mName + ":", &mWallParamItems);
            addCheckBox(wallSelGrid, par->mValue, &mWallParamItems, i);
        }
        else if (sourceList[i].mType == eParamEnum)
        {
            CParamEnum*   par = (CParamEnum*)param;
            addLabel(wallSelGrid, sourceList[i].mName + ":", &mWallParamItems);
            addComboBox(wallSelGrid, sourceList[i].mValues, &mWallParamItems, i, par->mValue);
        }
        else if (sourceList[i].mType == eParamStack)
        {
            CObjectStack*   stack = map.findObjectsStack(pos, side);
            addStackParamsItems(stack, sourceList.size() - 1, wallSelGrid, &mWallParamItems);
        }
        else if (sourceList[i].mType == eParamString)
        {
            CParamString*   par = (CParamString*)param;
            addLabel(wallSelGrid, sourceList[i].mName + ":", &mWallParamItems);
            addTextField(wallSelGrid, QString::fromStdString(par->mValue), &mWallParamItems, i);
        }
        else if (sourceList[i].mType == eParamScript)
        {
            CParamScript*   par = (CParamScript*)param;
            addLabel(wallSelGrid, sourceList[i].mName + ":", &mWallParamItems);
            addScriptField(wallSelGrid, QString(par->mValue.c_str()), &mWallParamItems, i);
        }
    }
}

//---------------------------------------------------------------------------------------------
void    CEditor::addObjParamsItems()
{
    CVec2           pos = mSelectStart / (TILE_SIZE / 2);
    CObjectStack*   stack = map.findObjectsStack(pos, eWallSideMax);

    addStackParamsItems(stack, 0, objSelGrid, &mObjParamItems);
}

//---------------------------------------------------------------------------------------------
void    CEditor::addStackParamsItems(CObjectStack* stack, int startId, QQuickItem* parent, std::vector<QQuickItem *> *list)
{
    size_t          size = 0;

    if (stack != NULL)
    {
        for (size_t i = 0; i < stack->getSize(); ++i)
        {
            CObject&    obj = stack->getObject(i);
            int         objType = obj.getType();

            addButton(parent, "delete", list, i);
            addComboBox(parent, bridge.itemsList, list, startId + i, objType);

            if (objType != 0)
            {
                std::vector<CParamType>& sourceList = CMap::mObjectsParams[objType];

                for (size_t j = 0; j < sourceList.size(); ++j)
                {
                    CParam* param = obj.mParams[j];

                    if (sourceList[j].mType == eParamString)
                    {
                        CParamString*   par = (CParamString*)param;
                        addLabel(parent, sourceList[j].mName + ":", list);
                        addTextField(parent, QString::fromStdString(par->mValue), list, (j + 1) * 0x10000 + i);
                    }
                    else if (sourceList[j].mType == eParamInt)
                    {
                        CParamInt*   par = (CParamInt*)param;
                        addLabel(parent, sourceList[j].mName + ":", list);
                        addTextField(parent, QString::number(par->mValue), list, (j + 1) * 0x10000 + i);
                    }
                }
            }
        }
        size = stack->getSize();
    }
    addButton(parent, "add", list, size);
}

//---------------------------------------------------------------------------------------------
void    CEditor::toolSelect()
{
    if (bridge.leftPressed == true)
    {
        mSelectEnd = bridge.mMousePos;

        if (bridge.leftPressing == true)
            mSelectStart = mSelectEnd;

        // display infos in the selection groupbox
        CRect   rectPixel = getSelectionRectPixel();
        CRect   rect = CRect(rectPixel.tl / TILE_SIZE, rectPixel.br / TILE_SIZE);
        QString positionText = getPositionString(rectPixel.tl, bridge.mTabIndex);
        bridge.set_selPosition(positionText);
        CVec2 selSize = rect.br - rect.tl + CVec2(1, 1);
        static char sizeText[256];
        sprintf(sizeText, "(%d,%d)", selSize.x, selSize.y);
        bridge.set_selSize(QString(sizeText));

        switch (bridge.mTabIndex)
        {
        case eTabTiles:
            if (selSize.x == 1 && selSize.y == 1)
            {
                uint8_t     type = map.getTile(rect.tl)->getType();
                bridge.set_selTileType(type);

                deleteParamItems(&mTileParamItems);
                addTileParamsItems(type);
            }
            break;

        case eTabWalls:
            if (isSelectSingleWall() == true)
            {
                EWallSide   side = getWallSideAbs(mSelectStart);
                uint8_t     type = map.getTile(rect.tl)->mWalls[side].getType();
                bridge.set_selWallType(type);

                deleteParamItems(&mWallParamItems);
                addWallParamsItems(type);
            }
            break;

        case eTabObjects:
            if (isSelectSingleObject() == true)
            {
                deleteParamItems(&mObjParamItems);
                addObjParamsItems();
            }
            break;

        default:
            break;
        }

        bridge.updateQMLImage();
    }
}

//---------------------------------------------------------------------------------------------
void    CEditor::readTilesDB(QString fileName)
{
    QDomDocument doc;

    QFile f(fileName);
    f.open(QIODevice::ReadOnly);
    doc.setContent(&f);
    f.close();

    QDomElement root = doc.documentElement();
    QDomElement tile = root.firstChild().toElement();

    while(!tile.isNull())
    {
        std::vector<CParamType> paramsTypesList;

        if (tile.tagName() == "tile")
        {
            CTileData   newTile;

            newTile.name = tile.attribute("name");

            QDomElement tileInfo = tile.firstChild().toElement();

            while(!tileInfo.isNull())
            {
                if (tileInfo.tagName() == "image")
                {
                    newTile.image = QString("tiles/") + tileInfo.text();
                }
                else if (tileInfo.tagName() == "param")
                {
                    QString paramType = tileInfo.attribute("type");

                    CParamType  newParamType;

                    if (paramType == "ornate")
                    {
                        newParamType.mType = eParamOrnate;
                        newParamType.mName = tileInfo.text();
                        paramsTypesList.push_back(newParamType);
                    }
                    else if (paramType == "int")
                    {
                        newParamType.mType = eParamInt;
                        newParamType.mName = tileInfo.text();
                        paramsTypesList.push_back(newParamType);
                    }
                    else if (paramType == "bool")
                    {
                        newParamType.mType = eParamBool;
                        newParamType.mName = tileInfo.text();
                        paramsTypesList.push_back(newParamType);
                    }
                    else if (paramType == "door")
                    {
                        newParamType.mType = eParamDoor;
                        newParamType.mName = tileInfo.text();
                        paramsTypesList.push_back(newParamType);
                    }
                    else if (paramType == "door_ornate")
                    {
                        newParamType.mType = eParamDoorOrnate;
                        newParamType.mName = tileInfo.text();
                        paramsTypesList.push_back(newParamType);
                    }
                    else if (paramType == "enum")
                    {
                        newParamType.mType = eParamEnum;
                        newParamType.mName = tileInfo.text();
                        newParamType.mValues = tileInfo.attribute("values").split(";");
                        paramsTypesList.push_back(newParamType);
                    }
                    else if (paramType == "script")
                    {
                        newParamType.mType = eParamScript;
                        newParamType.mName = tileInfo.text();
                        paramsTypesList.push_back(newParamType);
                    }
                }

                tileInfo = tileInfo.nextSibling().toElement();
            }
            tilesDatas.push_back(newTile);
            CMap::mTilesParams.push_back(paramsTypesList);
        }
        tile = tile.nextSibling().toElement();
    }
}

//---------------------------------------------------------------------------------------------
void    CEditor::readWallsDB(QString fileName)
{
    QDomDocument doc;

    QFile f(fileName);
    f.open(QIODevice::ReadOnly);
    doc.setContent(&f);
    f.close();

    QDomElement root = doc.documentElement();
    QDomElement wall = root.firstChild().toElement();

    while(!wall.isNull())
    {
        std::vector<CParamType> paramsTypesList;

        if (wall.tagName() == "wall")
        {
            CWallData   newWall;

            newWall.name = wall.attribute("name");

            QDomElement wallInfo = wall.firstChild().toElement();

            while(!wallInfo.isNull())
            {
                if (wallInfo.tagName() == "image")
                {
                    newWall.image = QString("tiles/") + wallInfo.text();
                }
                else if (wallInfo.tagName() == "param")
                {
                    QString paramType = wallInfo.attribute("type");

                    CParamType  newParamType;

                    if (paramType == "ornate")
                    {
                        newParamType.mType = eParamOrnate;
                        newParamType.mName = wallInfo.text();
                        paramsTypesList.push_back(newParamType);
                    }
                    else if (paramType == "int")
                    {
                        newParamType.mType = eParamInt;
                        newParamType.mName = wallInfo.text();
                        paramsTypesList.push_back(newParamType);
                    }
                    else if (paramType == "bool")
                    {
                        newParamType.mType = eParamBool;
                        newParamType.mName = wallInfo.text();
                        paramsTypesList.push_back(newParamType);
                    }
                    else if (paramType == "enum")
                    {
                        newParamType.mType = eParamEnum;
                        newParamType.mName = wallInfo.text();
                        newParamType.mValues = wallInfo.attribute("values").split(";");
                        paramsTypesList.push_back(newParamType);
                    }
                    else if (paramType == "stack")
                    {
                        newParamType.mType = eParamStack;
                        paramsTypesList.push_back(newParamType);
                    }
                    else if (paramType == "string")
                    {
                        newParamType.mType = eParamString;
                        newParamType.mName = wallInfo.text();
                        paramsTypesList.push_back(newParamType);
                    }
                    else if (paramType == "script")
                    {
                        newParamType.mType = eParamScript;
                        newParamType.mName = wallInfo.text();
                        paramsTypesList.push_back(newParamType);
                    }
                }

                wallInfo = wallInfo.nextSibling().toElement();
            }
            wallsDatas.push_back(newWall);
            CMap::mWallsParams.push_back(paramsTypesList);
        }
        wall = wall.nextSibling().toElement();
    }
}

//---------------------------------------------------------------------------------------------
void    CEditor::readObjectsDB(QString fileName)
{
    QDomDocument doc;

    QFile f(fileName);
    f.open(QIODevice::ReadOnly);
    doc.setContent(&f);
    f.close();

    QDomElement root = doc.documentElement();
    QDomNode    itemNode = root.firstChild();
    int i = 0;

    while(!itemNode.isNull())
    {
        if (itemNode.isElement())
        {
            QDomElement item = itemNode.toElement();
            std::vector<CParamType> paramsTypesList;

            if (item.tagName() == "item")
            {
                std::string name = item.attribute("name").toLocal8Bit().constData();
                name = textsDB[name];
                bridge.itemsList.append(QString("%1 - %2").arg(QString::number(i), QString::fromStdString(name)));
                i++;

                QDomElement itemInfo = item.firstChild().toElement();

                while(!itemInfo.isNull())
                {
                    if (itemInfo.tagName() == "param")
                    {
                        QString paramType = itemInfo.attribute("type");

                        CParamType  newParamType;

                        if (paramType == "string")
                        {
                            newParamType.mType = eParamString;
                            newParamType.mName = itemInfo.text();
                            paramsTypesList.push_back(newParamType);
                        }
                        else if (paramType == "int")
                        {
                            newParamType.mType = eParamInt;
                            newParamType.mName = itemInfo.text();
                            paramsTypesList.push_back(newParamType);
                        }
                    }

                    itemInfo = itemInfo.nextSibling().toElement();
                }

                CMap::mObjectsParams.push_back(paramsTypesList);
            }
        }
        itemNode = itemNode.nextSibling();
    }
}

//---------------------------------------------------------------------------------------------
void CEditor::readTextsDB(QString fileName)
{
    QDomDocument doc;

    QFile f(fileName);
    f.open(QIODevice::ReadOnly);
    doc.setContent(&f);
    f.close();

    QDomElement root = doc.documentElement();
    QDomElement dbString = root.firstChild().toElement();

    while(!dbString.isNull())
    {
        if (dbString.tagName() == "text")
        {
            std::string id = dbString.attribute("id").toUtf8().constData();
            std::string text = dbString.text().toUtf8().constData();
            textsDB[id] = text;
        }
        dbString = dbString.nextSibling().toElement();
    }
}

//---------------------------------------------------------------------------------------------
void    CEditor::readOnlyNamesDB(QString fileName, QString tagName, QStringList* list)
{
    QDomDocument doc;

    QFile f(fileName);
    f.open(QIODevice::ReadOnly);
    doc.setContent(&f);
    f.close();

    QDomElement root = doc.documentElement();
    QDomElement element = root.firstChild().toElement();

    int i = 0;
    while(!element.isNull())
    {
        if (element.tagName() == tagName)
        {
            list->append(QString("%1 - %2").arg(QString::number(i), element.attribute("name")));
            i++;
        }
        element = element.nextSibling().toElement();
    }
}

//---------------------------------------------------------------------------------------------
void    CEditor::initLists()
{
    readTextsDB("databases/texts.xml");

    readTilesDB("databases/tiles.xml");

    for (size_t i = 0; i < tilesDatas.size(); ++i)
        bridge.tilesList.append(QString("%1 - %2").arg(QString::number(i), tilesDatas[i].name));

    readWallsDB("databases/walls.xml");

    for (size_t i = 0; i < wallsDatas.size(); ++i)
        bridge.wallsList.append(QString("%1 - %2").arg(QString::number(i), wallsDatas[i].name));

    readObjectsDB("databases/items.xml");

    readOnlyNamesDB("databases/ornates.xml", "ornate", &bridge.ornatesList);
    readOnlyNamesDB("databases/floor_ornates.xml", "floor_ornate", &bridge.floorOrnatesList);
    readOnlyNamesDB("databases/doors.xml", "door", &bridge.doorsList);
    readOnlyNamesDB("databases/door_ornates.xml", "door_ornate", &bridge.doorOrnatesList);
}

//---------------------------------------------------------------------------------------------
QQuickItem* CEditor::loadQml(QString file, QQuickItem* parent)
{
    QQmlComponent   component(myQmlEngine, QUrl(file));
    QQuickItem*     object = qobject_cast<QQuickItem*>(component.create());
    QQmlEngine::setObjectOwnership(object, QQmlEngine::CppOwnership);
    object->setParent(parent);
    object->setParentItem(parent);
    return object;
}

//---------------------------------------------------------------------------------------------
void CEditor::addLabel(QQuickItem* parent, QString text, std::vector<QQuickItem *> *list)
{
    QQuickItem* object = loadQml("qrc:/ParamLabel.qml", parent);
    object->setProperty("text", QVariant(text));
    list->push_back(object);
}

//---------------------------------------------------------------------------------------------
void CEditor::addComboBox(QQuickItem* parent, QStringList& values, std::vector<QQuickItem*>* list, int id, int index)
{
    QQuickItem* object = loadQml("qrc:/ParamComboBox.qml", parent);
    object->setProperty("model", QVariant(values));
    object->setProperty("identifier", QVariant(id));
    object->setProperty("currentIndex", QVariant(index));
    list->push_back(object);
}

//---------------------------------------------------------------------------------------------
void CEditor::addTextField(QQuickItem* parent, QString text, std::vector<QQuickItem*>* list, int id)
{
    QQuickItem* object = loadQml("qrc:/ParamTextField.qml", parent);
    object->setProperty("text", QVariant(text));
    object->setProperty("identifier", QVariant(id));
    list->push_back(object);
}

//---------------------------------------------------------------------------------------------
void CEditor::addScriptField(QQuickItem* parent, QString text, std::vector<QQuickItem*>* list, int id)
{
    QQuickItem* object = loadQml("qrc:/ParamScriptField.qml", parent);
    object->setProperty("text", QVariant(text));
    object->setProperty("identifier", QVariant(id));
    list->push_back(object);
}

//---------------------------------------------------------------------------------------------
void CEditor::addCheckBox(QQuickItem* parent, bool isChecked, std::vector<QQuickItem*>* list, int id)
{
    QQuickItem* object = loadQml("qrc:/ParamCheckBox.qml", parent);
    object->setProperty("checked", QVariant(isChecked));
    object->setProperty("identifier", QVariant(id));
    list->push_back(object);
}

//---------------------------------------------------------------------------------------------
void CEditor::addButton(QQuickItem* parent, QString text, std::vector<QQuickItem *> *list, int id)
{
    QQuickItem* object = loadQml("qrc:/ParamButton.qml", parent);
    object->setProperty("text", QVariant(text));
    object->setProperty("identifier", QVariant(id));
    list->push_back(object);
}
